
import 'dart:async';
import 'package:flutter_adserver/model/ads_response.dart';
import 'package:flutter_adserver/repository/ads_repository.dart';
import 'package:get/get.dart';
import 'package:jwt_decode/jwt_decode.dart';

import '../ads.dart';

class AdViewController extends GetxController with StateMixin<AdsResponse>{

  String? adSlot;
  AdsRepository adsRepository = AdsRepository();

  @override
  void onClose() {
    timer?.cancel();
    super.onClose();
  }


  @override
  void onInit() {
    super.onInit();
  }

  void fetchAd(String? slotId) async{
    adSlot = slotId;
    if(Ads.config!.apiConfig!.baseUrl.isEmpty) {
      change(null, status: RxStatus.error("Base url not initialized"));
      throw("Base url not initialized");
    }
    if(Ads.config!.userConfig!.reseller.isEmpty) {
      change(null, status: RxStatus.error("Reseller id not initialized"));
      throw("Reseller id not initialized");
    }
      if(Ads.config!.userConfig!.subscriber.isEmpty) {
        change(null, status: RxStatus.error("Subscriber id not initialized"));
        throw("Subscriber id not initialized");
      }

      if(slotId == null){
        change(null, status: RxStatus.error("Slot id cannot be empty"));
        printError(info:"Slot id cannot be empty   ");
      }

      adsRepository.getBannerAds(slotId!).then((value){
        if(value != null){
          change(value, status: RxStatus.success());
          autoRefresh(value.token());
        }else{
          change(null, status: RxStatus.empty());
        }
      });
  }

  void impression(AdsResponse state) {
    if(state.impressionUrl().isNotEmpty){
      adsRepository.setImpression(state.impressionUrl());
    }
  }

  Timer? timer;

  void autoRefresh(String? token){
    if(token != null){
      var jwt = Jwt.parseJwt(token);
      var expiry = jwt["exp"];
      if(expiry != null){
        var date = DateTime.fromMillisecondsSinceEpoch(expiry*1000);
        var nextRefreshTime = date.millisecondsSinceEpoch - (DateTime.now().millisecondsSinceEpoch);
        if(nextRefreshTime > 5000) {
          startnextRefresh((nextRefreshTime * 0.80).toInt());
        }
      }
    }else{
      startnextRefresh(300000);
    }
  }

  void startnextRefresh(int nextRefreshTime) async{
    timer ??= Timer(Duration(milliseconds: nextRefreshTime), (){
          fetchAd(adSlot);
          timer = null;
      });
  }

}